(function() {
  var Nomplate;
  var __slice = Array.prototype.slice;
  Nomplate = (function() {
    function Nomplate() {
      this.output = '';
      this.pretty = true;
      this.indents = 0;
      this.indentation = '  ';
      this.carriageReturn = '\n';
    }
    Nomplate.prototype.node = function() {
      var args, attributes, collapsed, handler, name, value, _ref;
      name = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      _ref = this.processArgs(args), attributes = _ref[0], value = _ref[1], handler = _ref[2];
      this.writeIndentation();
      this.writeOpener(name, attributes);
      collapsed = typeof this.collapsible == "function" ? this.collapsible(name) : void 0;
      if (!collapsed) {
        this.write('>');
      }
      if (!collapsed && this.pretty && !value && handler) {
        this.writeUnindentation();
      }
      if (value) {
        this.write(value);
      }
      if (handler) {
        this.processHandler(handler);
        this.afterHandler();
      }
      if (collapsed) {
        this.write(' />');
        return;
      }
      if (handler) {
        this.writeIndentation();
      }
      this.writeCloser(name);
      return this.afterCloser();
    };
    Nomplate.prototype.collapsible = function(name) {
      return this.collapsibleNodes && this.collapsibleNodes.indexOf(name) > -1;
    };
    Nomplate.prototype.processArgs = function(args) {
      var attributes, handler, item, len, value, _i, _len;
      attributes = null;
      value = null;
      handler = null;
      len = args.length;
      for (_i = 0, _len = args.length; _i < _len; _i++) {
        item = args[_i];
        if (typeof item === 'string') {
          value = item;
        } else if (typeof item === 'function') {
          handler = item;
        } else {
          attributes = item;
        }
      }
      return [attributes, value, handler];
    };
    Nomplate.prototype.processHandler = function(handler) {
      return handler.call(this);
    };
    Nomplate.prototype.writeIndentation = function() {
      var num, _ref, _results;
      if (this.pretty) {
        _results = [];
        for (num = 0, _ref = this.indents; (0 <= _ref ? num < _ref : num > _ref); (0 <= _ref ? num += 1 : num -= 1)) {
          _results.push(this.write(this.indentation));
        }
        return _results;
      }
    };
    Nomplate.prototype.writeOpener = function(name, attributes) {
      this.write('<' + name);
      return this.writeAttributes(attributes);
    };
    Nomplate.prototype.writeCloser = function(name) {
      return this.write('</' + name + '>');
    };
    Nomplate.prototype.writeUnindentation = function() {
      if (this.pretty) {
        this.indents++;
        return this.write(this.carriageReturn);
      }
    };
    Nomplate.prototype.afterHandler = function() {
      return this.indents--;
    };
    Nomplate.prototype.afterCloser = function() {
      if (this.pretty) {
        return this.write('\n');
      }
    };
    Nomplate.prototype.writeAttributes = function(attributes) {
      var key, rendered, value;
      rendered = [];
      if (attributes) {
        this.write(' ');
        for (key in attributes) {
          value = attributes[key];
          rendered.push(key + '="' + attributes[key] + '"');
        }
        return this.write(rendered.join(' '));
      } else {
        return this.write('');
      }
    };
    Nomplate.prototype.write = function(message) {
      this.output += message;
      return this.writeToStream(message);
    };
    Nomplate.prototype.writeToStream = function(message) {
      if (this.stream) {
        return this.stream.write(message);
      }
    };
    return Nomplate;
  })();
  exports.Nomplate = Nomplate;
}).call(this);
